import { ThemeExtension } from '@chakra-ui/react';

declare type WithTypeScaleProps = {
    /**
     * The multiplier that is used to generate the set of font sizes
     *
     * The typical range is from `1.067` to `1.618`
     *
     * @see Type-Scale Generator https://type-scale.com
     */
    scale: number;
    /**
     * Base line-height number in rem to help assign proper lineHeight to each font-size generated for defaults, and used to generate vertical spacing tokens.
     *
     * The `rem` value helps calculate the actual line height for each font size to keep with vertical rhythm practice.
     * @default 1.5
     */
    lineHeight?: number;
    /**
     * If true, the font sizes and line heights generated will use the `clamp()` function
     * instead of using an array with Chakra's breakpoint checking.
     *
     * Can also accept an object (making it truthy) to set the minimum and/or maximum viewport widths to clamp at. (`minVW` and `maxVW`)
     *
     * NOTE: This is only applied to the `sizes` theme object for the `Heading` component. The `Text` component theme does not use `sizes`.
     *
     * @default false
     */
    isClamped?: boolean | {
        minVW?: number;
        maxVW?: number;
    };
};
/**
 * Chakra UI theme extension to generate font sizes and line height via a specified scale.
 *
 * Uses the following set of token values for sizing:
 * `['sm', 'base', 'lg', 'xl', '2xl', '3xl', '4xl', '5xl']`
 *
 * - Generates a `vertical` token object for `space` theme with values from `1` - `13` (i.e. `marginB="vertical.1"`)
 * - Generates a set of `fontSize` tokens
 * - Generates `sizes` tokens for the `Heading` component and a default `marginBottom` value
 * - Generates `sizes` tokens for the `Text` component and a default `lineHeight` and `marginBottom` value.
 *
 * @return ThemeExtension
 *
 * @see https://github.com/TylerAPfledderer/chakra-ui-typescale#readme
 */
declare function withTypeScale(props: WithTypeScaleProps): ThemeExtension;

export { withTypeScale };
